# this file is @generated
import typing as t

from pydantic import ModelWrapValidatorHandler, model_validator
from typing_extensions import Self

from .amazon_s3_patch_config import AmazonS3PatchConfig
from .azure_blob_storage_patch_config import AzureBlobStoragePatchConfig
from .common import BaseModel
from .google_cloud_storage_patch_config import GoogleCloudStoragePatchConfig
from .http_patch_config import HttpPatchConfig
from .otel_tracing_patch_config import OtelTracingPatchConfig
from .sink_status_in import SinkStatusIn


class StreamSinkPatch(BaseModel):
    batch_size: t.Optional[int] = None

    event_types: t.Optional[t.List[str]] = None

    max_wait_secs: t.Optional[int] = None

    metadata: t.Optional[t.Dict[str, str]] = None

    status: t.Optional[SinkStatusIn] = None

    uid: t.Optional[str] = None
    """The StreamSink's UID."""

    type: t.Union[
        t.Literal["poller"],
        t.Literal["azureBlobStorage"],
        t.Literal["otelTracing"],
        t.Literal["http"],
        t.Literal["amazonS3"],
        t.Literal["googleCloudStorage"],
    ]
    config: t.Union[
        t.Dict[str, t.Any],
        AzureBlobStoragePatchConfig,
        OtelTracingPatchConfig,
        HttpPatchConfig,
        AmazonS3PatchConfig,
        GoogleCloudStoragePatchConfig,
    ]

    @model_validator(mode="wrap")
    @classmethod
    def validate_model(
        cls, data: t.Any, handler: ModelWrapValidatorHandler[Self]
    ) -> Self:
        if "config" not in data:
            data["config"] = {}
        output = handler(data)
        if output.type == "poller":
            output.config = data.get("config", {})
        elif output.type == "azureBlobStorage":
            output.config = AzureBlobStoragePatchConfig.model_validate(
                data.get("config", {})
            )
        elif output.type == "otelTracing":
            output.config = OtelTracingPatchConfig.model_validate(
                data.get("config", {})
            )
        elif output.type == "http":
            output.config = HttpPatchConfig.model_validate(data.get("config", {}))
        elif output.type == "amazonS3":
            output.config = AmazonS3PatchConfig.model_validate(data.get("config", {}))
        elif output.type == "googleCloudStorage":
            output.config = GoogleCloudStoragePatchConfig.model_validate(
                data.get("config", {})
            )
        else:
            raise ValueError(f"Unexpected type `{output.type}`")
        return output



# This file is @generated
import typing as t
from dataclasses import dataclass

from .. import models
from ..models import (
    EmptyResponse,
    EndpointSecretRotateIn,
    ListResponseStreamSinkOut,
    SinkSecretOut,
    SinkTransformIn,
    StreamSinkIn,
    StreamSinkOut,
    StreamSinkPatch,
)
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class StreamingSinkListOptions(BaseOptions):
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""
    order: t.Optional[models.Ordering] = None
    """The sorting order of the returned items"""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "limit": self.limit,
                "iterator": self.iterator,
                "order": self.order,
            }
        )


@dataclass
class StreamingSinkCreateOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class StreamingSinkRotateSecretOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


class StreamingSinkAsync(ApiBase):
    async def list(
        self,
        stream_id: str,
        options: StreamingSinkListOptions = StreamingSinkListOptions(),
    ) -> ListResponseStreamSinkOut:
        """List of all the stream's sinks."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/stream/{stream_id}/sink",
            path_params={
                "stream_id": stream_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseStreamSinkOut.model_validate(response.json())

    async def create(
        self,
        stream_id: str,
        stream_sink_in: StreamSinkIn,
        options: StreamingSinkCreateOptions = StreamingSinkCreateOptions(),
    ) -> StreamSinkOut:
        """Creates a new sink."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/stream/{stream_id}/sink",
            path_params={
                "stream_id": stream_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=stream_sink_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return StreamSinkOut.model_validate(response.json())

    async def get(self, stream_id: str, sink_id: str) -> StreamSinkOut:
        """Get a sink by id or uid."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
        )
        return StreamSinkOut.model_validate(response.json())

    async def update(
        self, stream_id: str, sink_id: str, stream_sink_in: StreamSinkIn
    ) -> StreamSinkOut:
        """Update a sink."""
        response = await self._request_asyncio(
            method="put",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            json_body=stream_sink_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return StreamSinkOut.model_validate(response.json())

    async def delete(self, stream_id: str, sink_id: str) -> None:
        """Delete a sink."""
        await self._request_asyncio(
            method="delete",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
        )

    async def patch(
        self, stream_id: str, sink_id: str, stream_sink_patch: StreamSinkPatch
    ) -> StreamSinkOut:
        """Partially update a sink."""
        response = await self._request_asyncio(
            method="patch",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            json_body=stream_sink_patch.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return StreamSinkOut.model_validate(response.json())

    async def get_secret(self, stream_id: str, sink_id: str) -> SinkSecretOut:
        """Get the sink's signing secret (only supported for http sinks)

        This is used to verify the authenticity of the delivery.

        For more information please refer to [the consuming webhooks docs](https://docs.svix.com/consuming-webhooks/)."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/secret",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
        )
        return SinkSecretOut.model_validate(response.json())

    async def rotate_secret(
        self,
        stream_id: str,
        sink_id: str,
        endpoint_secret_rotate_in: EndpointSecretRotateIn,
        options: StreamingSinkRotateSecretOptions = StreamingSinkRotateSecretOptions(),
    ) -> EmptyResponse:
        """Rotates the signing secret (only supported for http sinks)."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/secret/rotate",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=endpoint_secret_rotate_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return EmptyResponse.model_validate(response.json())

    async def transformation_partial_update(
        self, stream_id: str, sink_id: str, sink_transform_in: SinkTransformIn
    ) -> EmptyResponse:
        """Set or unset the transformation code associated with this sink."""
        response = await self._request_asyncio(
            method="patch",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/transformation",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            json_body=sink_transform_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return EmptyResponse.model_validate(response.json())


class StreamingSink(ApiBase):
    def list(
        self,
        stream_id: str,
        options: StreamingSinkListOptions = StreamingSinkListOptions(),
    ) -> ListResponseStreamSinkOut:
        """List of all the stream's sinks."""
        response = self._request_sync(
            method="get",
            path="/api/v1/stream/{stream_id}/sink",
            path_params={
                "stream_id": stream_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseStreamSinkOut.model_validate(response.json())

    def create(
        self,
        stream_id: str,
        stream_sink_in: StreamSinkIn,
        options: StreamingSinkCreateOptions = StreamingSinkCreateOptions(),
    ) -> StreamSinkOut:
        """Creates a new sink."""
        response = self._request_sync(
            method="post",
            path="/api/v1/stream/{stream_id}/sink",
            path_params={
                "stream_id": stream_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=stream_sink_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return StreamSinkOut.model_validate(response.json())

    def get(self, stream_id: str, sink_id: str) -> StreamSinkOut:
        """Get a sink by id or uid."""
        response = self._request_sync(
            method="get",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
        )
        return StreamSinkOut.model_validate(response.json())

    def update(
        self, stream_id: str, sink_id: str, stream_sink_in: StreamSinkIn
    ) -> StreamSinkOut:
        """Update a sink."""
        response = self._request_sync(
            method="put",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            json_body=stream_sink_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return StreamSinkOut.model_validate(response.json())

    def delete(self, stream_id: str, sink_id: str) -> None:
        """Delete a sink."""
        self._request_sync(
            method="delete",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
        )

    def patch(
        self, stream_id: str, sink_id: str, stream_sink_patch: StreamSinkPatch
    ) -> StreamSinkOut:
        """Partially update a sink."""
        response = self._request_sync(
            method="patch",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            json_body=stream_sink_patch.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return StreamSinkOut.model_validate(response.json())

    def get_secret(self, stream_id: str, sink_id: str) -> SinkSecretOut:
        """Get the sink's signing secret (only supported for http sinks)

        This is used to verify the authenticity of the delivery.

        For more information please refer to [the consuming webhooks docs](https://docs.svix.com/consuming-webhooks/)."""
        response = self._request_sync(
            method="get",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/secret",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
        )
        return SinkSecretOut.model_validate(response.json())

    def rotate_secret(
        self,
        stream_id: str,
        sink_id: str,
        endpoint_secret_rotate_in: EndpointSecretRotateIn,
        options: StreamingSinkRotateSecretOptions = StreamingSinkRotateSecretOptions(),
    ) -> EmptyResponse:
        """Rotates the signing secret (only supported for http sinks)."""
        response = self._request_sync(
            method="post",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/secret/rotate",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=endpoint_secret_rotate_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return EmptyResponse.model_validate(response.json())

    def transformation_partial_update(
        self, stream_id: str, sink_id: str, sink_transform_in: SinkTransformIn
    ) -> EmptyResponse:
        """Set or unset the transformation code associated with this sink."""
        response = self._request_sync(
            method="patch",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/transformation",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            json_body=sink_transform_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return EmptyResponse.model_validate(response.json())



# this file is @generated
import typing as t
from datetime import datetime

from .common import BaseModel
from .connector_kind import ConnectorKind
from .connector_product import ConnectorProduct


class ConnectorOut(BaseModel):
    allowed_event_types: t.Optional[t.List[str]] = None

    created_at: datetime

    description: str

    feature_flags: t.Optional[t.List[str]] = None

    id: str
    """The Connector's ID."""

    instructions: str

    kind: ConnectorKind

    logo: t.Optional[str] = None

    name: str

    org_id: str
    """The Environment's ID."""

    product_type: ConnectorProduct

    transformation: str

    transformation_updated_at: datetime

    uid: t.Optional[str] = None
    """The Connector's UID."""

    updated_at: datetime



# This file is @generated
from ..models import EndpointHeadersOut, HttpSinkHeadersPatchIn, SinkTransformationOut
from .common import ApiBase
from .streaming_event_type import (
    StreamingEventType,
    StreamingEventTypeAsync,
)
from .streaming_events import (
    StreamingEvents,
    StreamingEventsAsync,
)
from .streaming_sink import (
    StreamingSink,
    StreamingSinkAsync,
)
from .streaming_stream import (
    StreamingStream,
    StreamingStreamAsync,
)


class StreamingAsync(ApiBase):
    @property
    def event_type(self) -> StreamingEventTypeAsync:
        return StreamingEventTypeAsync(self._client)

    @property
    def events(self) -> StreamingEventsAsync:
        return StreamingEventsAsync(self._client)

    @property
    def sink(self) -> StreamingSinkAsync:
        return StreamingSinkAsync(self._client)

    @property
    def stream(self) -> StreamingStreamAsync:
        return StreamingStreamAsync(self._client)

    async def sink_headers_get(
        self, stream_id: str, sink_id: str
    ) -> EndpointHeadersOut:
        """Get the HTTP sink headers. Only valid for `http` or `otelTracing` sinks."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/headers",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
        )
        return EndpointHeadersOut.model_validate(response.json())

    async def sink_headers_patch(
        self,
        stream_id: str,
        sink_id: str,
        http_sink_headers_patch_in: HttpSinkHeadersPatchIn,
    ) -> EndpointHeadersOut:
        """Updates the Sink's headers. Only valid for `http` or `otelTracing` sinks."""
        response = await self._request_asyncio(
            method="patch",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/headers",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            json_body=http_sink_headers_patch_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return EndpointHeadersOut.model_validate(response.json())

    async def sink_transformation_get(
        self, stream_id: str, sink_id: str
    ) -> SinkTransformationOut:
        """Get the transformation code associated with this sink."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/transformation",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
        )
        return SinkTransformationOut.model_validate(response.json())


class Streaming(ApiBase):
    @property
    def event_type(self) -> StreamingEventType:
        return StreamingEventType(self._client)

    @property
    def events(self) -> StreamingEvents:
        return StreamingEvents(self._client)

    @property
    def sink(self) -> StreamingSink:
        return StreamingSink(self._client)

    @property
    def stream(self) -> StreamingStream:
        return StreamingStream(self._client)

    def sink_headers_get(self, stream_id: str, sink_id: str) -> EndpointHeadersOut:
        """Get the HTTP sink headers. Only valid for `http` or `otelTracing` sinks."""
        response = self._request_sync(
            method="get",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/headers",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
        )
        return EndpointHeadersOut.model_validate(response.json())

    def sink_headers_patch(
        self,
        stream_id: str,
        sink_id: str,
        http_sink_headers_patch_in: HttpSinkHeadersPatchIn,
    ) -> EndpointHeadersOut:
        """Updates the Sink's headers. Only valid for `http` or `otelTracing` sinks."""
        response = self._request_sync(
            method="patch",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/headers",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            json_body=http_sink_headers_patch_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return EndpointHeadersOut.model_validate(response.json())

    def sink_transformation_get(
        self, stream_id: str, sink_id: str
    ) -> SinkTransformationOut:
        """Get the transformation code associated with this sink."""
        response = self._request_sync(
            method="get",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/transformation",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
        )
        return SinkTransformationOut.model_validate(response.json())



# this file is @generated
import typing as t
from datetime import datetime

from .common import BaseModel


class StreamOut(BaseModel):
    created_at: datetime

    id: str
    """The stream's ID."""

    metadata: t.Dict[str, str]

    name: t.Optional[str] = None
    """The stream's name."""

    uid: t.Optional[str] = None
    """The stream's UID."""

    updated_at: datetime



# this file is @generated
import typing as t

from .common import BaseModel


class SinkHttpConfig(BaseModel):
    headers: t.Optional[t.Dict[str, str]] = None

    key: t.Optional[str] = None

    url: str



# this file is @generated
import typing as t

from .common import BaseModel


class ApplicationTokenExpireIn(BaseModel):
    expiry: t.Optional[int] = None
    """How many seconds until the old key is expired."""

    session_ids: t.Optional[t.List[str]] = None
    """An optional list of session ids.

    If any session ids are specified, only Application tokens created with that session id will be expired."""



# this file is @generated
from enum import Enum


class AppPortalCapability(str, Enum):
    VIEW_BASE = "ViewBase"
    VIEW_ENDPOINT_SECRET = "ViewEndpointSecret"
    MANAGE_ENDPOINT_SECRET = "ManageEndpointSecret"
    MANAGE_TRANSFORMATIONS = "ManageTransformations"
    CREATE_ATTEMPTS = "CreateAttempts"
    MANAGE_ENDPOINT = "ManageEndpoint"

    def __str__(self) -> str:
        return str(self.value)



# this file is @generated
import typing as t

from .common import BaseModel


class EndpointHeadersPatchIn(BaseModel):
    delete_headers: t.Optional[t.List[str]] = None
    """A list of headers be be removed"""

    headers: t.Dict[str, str]



# this file is @generated
import typing as t

from .common import BaseModel


class RotatePollerTokenIn(BaseModel):
    expiry: t.Optional[int] = None
    """How long the token will be valid for, in seconds. Can be up to 31,536,000 seconds (1 year)."""

    old_token_expiry: t.Optional[int] = None
    """Updates the previous token's expiration, in seconds.

    If set to 0, the old token will immediately be revoked. Must be between 0 and 86,400 seconds (1 day).

    Defaults to 300 seconds (5 minutes)."""



# this file is @generated
import typing as t
from datetime import datetime

from .common import BaseModel
from .http_attempt_times import HttpAttemptTimes
from .message_status import MessageStatus


class MessageAttemptLog(BaseModel):
    app_id: str
    """The Application's ID."""

    app_uid: t.Optional[str] = None
    """The Application's UID."""

    attempt_count: int

    attempt_end: datetime

    attempt_id: str
    """The MessageAttempt's ID."""

    attempt_start: datetime

    endpoint_id: str
    """The Endpoint's ID."""

    event_type: t.Optional[str] = None
    """The event type's name"""

    http_times: t.Optional[HttpAttemptTimes] = None

    msg_created: datetime

    msg_event_id: t.Optional[str] = None
    """The Message's UID."""

    msg_id: str
    """The Message's ID."""

    org_id: str
    """The Environment's ID."""

    response_status_code: int

    status: MessageStatus



# this file is @generated
import typing as t

from .common import BaseModel
from .connector_kind import ConnectorKind


class ConnectorUpdate(BaseModel):
    allowed_event_types: t.Optional[t.List[str]] = None

    description: t.Optional[str] = None

    feature_flags: t.Optional[t.List[str]] = None

    instructions: t.Optional[str] = None

    kind: t.Optional[ConnectorKind] = None

    logo: t.Optional[str] = None

    name: t.Optional[str] = None

    transformation: str



# this file is @generated

from .common import BaseModel
from .ingest_message_attempt_failing_event_data import (
    IngestMessageAttemptFailingEventData,
)


class IngestMessageAttemptFailingEvent(BaseModel):
    """Sent after a message has been failing for a few times.
    It's sent on the fourth failure. It complements `ingest.message.attempt.exhausted` which is sent after the last failure."""

    data: IngestMessageAttemptFailingEventData

    type: str



# This file is @generated
import typing as t
from dataclasses import dataclass
from datetime import datetime

from ..models import CreateStreamEventsIn, CreateStreamEventsOut, EventStreamOut
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class StreamingEventsCreateOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class StreamingEventsGetOptions(BaseOptions):
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""
    after: t.Optional[datetime] = None

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "limit": self.limit,
                "iterator": self.iterator,
                "after": self.after,
            }
        )


class StreamingEventsAsync(ApiBase):
    async def create(
        self,
        stream_id: str,
        create_stream_events_in: CreateStreamEventsIn,
        options: StreamingEventsCreateOptions = StreamingEventsCreateOptions(),
    ) -> CreateStreamEventsOut:
        """Creates events on the Stream."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/stream/{stream_id}/events",
            path_params={
                "stream_id": stream_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=create_stream_events_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return CreateStreamEventsOut.model_validate(response.json())

    async def get(
        self,
        stream_id: str,
        sink_id: str,
        options: StreamingEventsGetOptions = StreamingEventsGetOptions(),
    ) -> EventStreamOut:
        """Iterate over a stream of events.

        The sink must be of type `poller` to use the poller endpoint."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/events",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return EventStreamOut.model_validate(response.json())


class StreamingEvents(ApiBase):
    def create(
        self,
        stream_id: str,
        create_stream_events_in: CreateStreamEventsIn,
        options: StreamingEventsCreateOptions = StreamingEventsCreateOptions(),
    ) -> CreateStreamEventsOut:
        """Creates events on the Stream."""
        response = self._request_sync(
            method="post",
            path="/api/v1/stream/{stream_id}/events",
            path_params={
                "stream_id": stream_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=create_stream_events_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return CreateStreamEventsOut.model_validate(response.json())

    def get(
        self,
        stream_id: str,
        sink_id: str,
        options: StreamingEventsGetOptions = StreamingEventsGetOptions(),
    ) -> EventStreamOut:
        """Iterate over a stream of events.

        The sink must be of type `poller` to use the poller endpoint."""
        response = self._request_sync(
            method="get",
            path="/api/v1/stream/{stream_id}/sink/{sink_id}/events",
            path_params={
                "stream_id": stream_id,
                "sink_id": sink_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return EventStreamOut.model_validate(response.json())



# this file is @generated
import typing as t

from .common import BaseModel
from .event_out import EventOut


class EventStreamOut(BaseModel):
    data: t.List[EventOut]

    done: bool

    iterator: str



# this file is @generated

from .common import BaseModel
from .ingest_message_attempt_recovered_event_data import (
    IngestMessageAttemptRecoveredEventData,
)


class IngestMessageAttemptRecoveredEvent(BaseModel):
    """Sent on a successful dispatch after an earlier failure op webhook has already been sent."""

    data: IngestMessageAttemptRecoveredEventData

    type: str



# this file is @generated
import typing as t
from datetime import datetime

from .common import BaseModel


class EndpointTransformationOut(BaseModel):
    code: t.Optional[str] = None

    enabled: t.Optional[bool] = None

    updated_at: t.Optional[datetime] = None



# this file is @generated
import typing as t

from .common import BaseModel


class AzureBlobStoragePatchConfig(BaseModel):
    access_key: t.Optional[str] = None

    account: t.Optional[str] = None

    container: t.Optional[str] = None



# this file is @generated
from enum import Enum


class SinkStatus(str, Enum):
    ENABLED = "enabled"
    PAUSED = "paused"
    DISABLED = "disabled"
    RETRYING = "retrying"

    def __str__(self) -> str:
        return str(self.value)



# this file is @generated
import typing as t

from pydantic import ModelWrapValidatorHandler, model_validator
from typing_extensions import Self

from .azure_blob_storage_config import AzureBlobStorageConfig
from .common import BaseModel
from .google_cloud_storage_config import GoogleCloudStorageConfig
from .s3_config import S3Config
from .sink_http_config import SinkHttpConfig
from .sink_otel_v1_config import SinkOtelV1Config
from .sink_status_in import SinkStatusIn


class StreamSinkIn(BaseModel):
    batch_size: t.Optional[int] = None
    """How many events will be batched in a request to the Sink."""

    event_types: t.Optional[t.List[str]] = None
    """A list of event types that filter which events are dispatched to the Sink. An empty list (or null) will not filter out any events."""

    max_wait_secs: t.Optional[int] = None
    """How long to wait before a batch of events is sent, if the `batchSize` is not reached.

    For example, with a `batchSize` of 100 and `maxWaitSecs` of 10, we will send a request after 10 seconds or 100 events, whichever comes first.

    Note that we will never send an empty batch of events to the Sink."""

    metadata: t.Optional[t.Dict[str, str]] = None

    status: t.Optional[SinkStatusIn] = None
    """Whether the sink will receive events.

    If the sink is `enabled`, any events posted to the stream will be dispatched to the Sink in the same order that events were posted to the stream.

    If the sink is `disabled`, events will not be dispatched to the sink until the sink is reenabled."""

    uid: t.Optional[str] = None
    """An optional unique identifier for the sink."""

    type: t.Union[
        t.Literal["poller"],
        t.Literal["azureBlobStorage"],
        t.Literal["otelTracing"],
        t.Literal["http"],
        t.Literal["amazonS3"],
        t.Literal["googleCloudStorage"],
    ]
    config: t.Union[
        t.Dict[str, t.Any],
        AzureBlobStorageConfig,
        SinkOtelV1Config,
        SinkHttpConfig,
        S3Config,
        GoogleCloudStorageConfig,
    ]

    @model_validator(mode="wrap")
    @classmethod
    def validate_model(
        cls, data: t.Any, handler: ModelWrapValidatorHandler[Self]
    ) -> Self:
        if "config" not in data:
            data["config"] = {}
        output = handler(data)
        if output.type == "poller":
            output.config = data.get("config", {})
        elif output.type == "azureBlobStorage":
            output.config = AzureBlobStorageConfig.model_validate(
                data.get("config", {})
            )
        elif output.type == "otelTracing":
            output.config = SinkOtelV1Config.model_validate(data.get("config", {}))
        elif output.type == "http":
            output.config = SinkHttpConfig.model_validate(data.get("config", {}))
        elif output.type == "amazonS3":
            output.config = S3Config.model_validate(data.get("config", {}))
        elif output.type == "googleCloudStorage":
            output.config = GoogleCloudStorageConfig.model_validate(
                data.get("config", {})
            )
        else:
            raise ValueError(f"Unexpected type `{output.type}`")
        return output



# this file is @generated
import typing as t

from .common import BaseModel
from .message_attempt_log import MessageAttemptLog


class MessageAttemptLogEvent(BaseModel):
    """Sent after message attempts are made. Contains metadata about message attempts and their results. In order to reduce the frequency of webhooks, these are sent in batches periodically."""

    data: t.List[MessageAttemptLog]

    type: str



# This file is @generated
import typing as t
from dataclasses import dataclass

from .. import models
from ..models import (
    ListResponseStreamEventTypeOut,
    StreamEventTypeIn,
    StreamEventTypeOut,
    StreamEventTypePatch,
)
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class StreamingEventTypeListOptions(BaseOptions):
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""
    order: t.Optional[models.Ordering] = None
    """The sorting order of the returned items"""
    include_archived: t.Optional[bool] = None
    """Include archived (deleted but not expunged) items in the response."""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "limit": self.limit,
                "iterator": self.iterator,
                "order": self.order,
                "include_archived": self.include_archived,
            }
        )


@dataclass
class StreamingEventTypeCreateOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class StreamingEventTypeDeleteOptions(BaseOptions):
    expunge: t.Optional[bool] = None
    """By default, event types are archived when "deleted". With this flag, they are deleted entirely."""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "expunge": self.expunge,
            }
        )


class StreamingEventTypeAsync(ApiBase):
    async def list(
        self, options: StreamingEventTypeListOptions = StreamingEventTypeListOptions()
    ) -> ListResponseStreamEventTypeOut:
        """List of all the organization's event types for streaming."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/stream/event-type",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseStreamEventTypeOut.model_validate(response.json())

    async def create(
        self,
        stream_event_type_in: StreamEventTypeIn,
        options: StreamingEventTypeCreateOptions = StreamingEventTypeCreateOptions(),
    ) -> StreamEventTypeOut:
        """Create an event type for Streams."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/stream/event-type",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=stream_event_type_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return StreamEventTypeOut.model_validate(response.json())

    async def get(self, name: str) -> StreamEventTypeOut:
        """Get an event type."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/stream/event-type/{name}",
            path_params={
                "name": name,
            },
        )
        return StreamEventTypeOut.model_validate(response.json())

    async def update(
        self, name: str, stream_event_type_in: StreamEventTypeIn
    ) -> StreamEventTypeOut:
        """Update or create a event type for Streams."""
        response = await self._request_asyncio(
            method="put",
            path="/api/v1/stream/event-type/{name}",
            path_params={
                "name": name,
            },
            json_body=stream_event_type_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return StreamEventTypeOut.model_validate(response.json())

    async def delete(
        self,
        name: str,
        options: StreamingEventTypeDeleteOptions = StreamingEventTypeDeleteOptions(),
    ) -> None:
        """Delete an event type."""
        await self._request_asyncio(
            method="delete",
            path="/api/v1/stream/event-type/{name}",
            path_params={
                "name": name,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )

    async def patch(
        self, name: str, stream_event_type_patch: StreamEventTypePatch
    ) -> StreamEventTypeOut:
        """Patch an event type for Streams."""
        response = await self._request_asyncio(
            method="patch",
            path="/api/v1/stream/event-type/{name}",
            path_params={
                "name": name,
            },
            json_body=stream_event_type_patch.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return StreamEventTypeOut.model_validate(response.json())


class StreamingEventType(ApiBase):
    def list(
        self, options: StreamingEventTypeListOptions = StreamingEventTypeListOptions()
    ) -> ListResponseStreamEventTypeOut:
        """List of all the organization's event types for streaming."""
        response = self._request_sync(
            method="get",
            path="/api/v1/stream/event-type",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseStreamEventTypeOut.model_validate(response.json())

    def create(
        self,
        stream_event_type_in: StreamEventTypeIn,
        options: StreamingEventTypeCreateOptions = StreamingEventTypeCreateOptions(),
    ) -> StreamEventTypeOut:
        """Create an event type for Streams."""
        response = self._request_sync(
            method="post",
            path="/api/v1/stream/event-type",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=stream_event_type_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return StreamEventTypeOut.model_validate(response.json())

    def get(self, name: str) -> StreamEventTypeOut:
        """Get an event type."""
        response = self._request_sync(
            method="get",
            path="/api/v1/stream/event-type/{name}",
            path_params={
                "name": name,
            },
        )
        return StreamEventTypeOut.model_validate(response.json())

    def update(
        self, name: str, stream_event_type_in: StreamEventTypeIn
    ) -> StreamEventTypeOut:
        """Update or create a event type for Streams."""
        response = self._request_sync(
            method="put",
            path="/api/v1/stream/event-type/{name}",
            path_params={
                "name": name,
            },
            json_body=stream_event_type_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return StreamEventTypeOut.model_validate(response.json())

    def delete(
        self,
        name: str,
        options: StreamingEventTypeDeleteOptions = StreamingEventTypeDeleteOptions(),
    ) -> None:
        """Delete an event type."""
        self._request_sync(
            method="delete",
            path="/api/v1/stream/event-type/{name}",
            path_params={
                "name": name,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )

    def patch(
        self, name: str, stream_event_type_patch: StreamEventTypePatch
    ) -> StreamEventTypeOut:
        """Patch an event type for Streams."""
        response = self._request_sync(
            method="patch",
            path="/api/v1/stream/event-type/{name}",
            path_params={
                "name": name,
            },
            json_body=stream_event_type_patch.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return StreamEventTypeOut.model_validate(response.json())



# this file is @generated

from .common import BaseModel
from .ingest_message_attempt_exhausted_event_data import (
    IngestMessageAttemptExhaustedEventData,
)


class IngestMessageAttemptExhaustedEvent(BaseModel):
    """Sent when a message delivery has failed (all of the retry attempts have been exhausted)."""

    data: IngestMessageAttemptExhaustedEventData

    type: str



import typing as t
from datetime import datetime
from standardwebhooks.webhooks import Webhook as StdWh
from standardwebhooks.exceptions import WebhookVerificationError


class Webhook:
    _inner: StdWh

    def __init__(self, whsecret: t.Union[str, bytes]):
        self._inner = StdWh(whsecret)

    def verify(self, data: t.Union[bytes, str], headers: t.Dict[str, str]) -> t.Any:
        headers = {k.lower(): v for k, v in headers.items()}

        headers["webhook-id"] = headers.get("svix-id", headers.get("webhook-id", ""))
        headers["webhook-signature"] = headers.get(
            "svix-signature", headers.get("webhook-signature", "")
        )
        headers["webhook-timestamp"] = headers.get(
            "svix-timestamp", headers.get("webhook-timestamp", "")
        )

        return self._inner.verify(data, headers)

    def sign(self, msg_id: str, timestamp: datetime, data: str) -> str:
        return self._inner.sign(msg_id, timestamp, data)


__all__ = ["Webhook", "WebhookVerificationError"]



# this file is @generated
import typing as t

from .common import BaseModel


class MessagePrecheckIn(BaseModel):
    channels: t.Optional[t.List[str]] = None

    event_type: str
    """The event type's name"""



# this file is @generated
import typing as t

from .common import BaseModel


class StreamIn(BaseModel):
    metadata: t.Optional[t.Dict[str, str]] = None

    name: str
    """The stream's name."""

    uid: t.Optional[str] = None
    """An optional unique identifier for the stream."""



# this file is @generated
import typing as t

from .common import BaseModel


class EventTypePatch(BaseModel):
    archived: t.Optional[bool] = None

    deprecated: t.Optional[bool] = None

    description: t.Optional[str] = None

    feature_flag: t.Optional[str] = None

    feature_flags: t.Optional[t.List[str]] = None

    group_name: t.Optional[str] = None
    """The event type group's name"""

    schemas: t.Optional[t.Dict[str, t.Any]] = None



# this file is @generated

from .common import BaseModel
from .ingest_endpoint_disabled_event_data import IngestEndpointDisabledEventData


class IngestEndpointDisabledEvent(BaseModel):
    """Sent when an ingest endpoint has been automatically disabled after continuous failures, or manually via an API call."""

    data: IngestEndpointDisabledEventData

    type: str
